@@ -32,6 +32,7 @@ class cudaFlow;
 class cudaFlowCapturer;
 class cudaFlowCapturerBase;
 class cudaCapturingBase;
+class cudaLinearCapturing;
 class cudaSequentialCapturing;
 class cudaRoundRobinCapturing;
 class cublasFlowCapturer;
@@ -42,6 +43,9 @@ class syclGraph;
 class syclTask;
 class syclFlow;
 
+// clas
+class Sanitizer;
+
 
 }  // end of namespace tf -----------------------------------------------------
 
